package de.hendriklipka.q3master;

import org.log4j.*;

import java.net.*;
import java.io.IOException;
import java.io.InterruptedIOException;

public class BroadcastQuery extends Thread
{
    private static Category CAT=Category.getInstance(BroadcastQuery.class.getName());
    private static Category LOG_CAT=Category.getInstance("q3master.broadcast");

    protected String[] _addresses=null;
    protected int _sleepTime=0;

    protected StatusHandler _handler=null;

    protected DatagramSocket _socket=null;

    public BroadcastQuery(String[] addresses, int sleepTime, StatusHandler handler)
    {
        _addresses=addresses;
        _sleepTime=sleepTime;
        _handler=handler;
    }

    public void run()
    {
        _socket=de.hendriklipka.util.UDPSocketPool.getSocket();
        try
        {
            _socket.setSoTimeout(1000);
        }
        catch (SocketException e)
        {
            CAT.error(e);
        }
        new BroadcastListener(false).start();
        while (true)
        {
            for (int i=0;i<_addresses.length;i++)
            {
                CAT.debug("sending broadcast packet to "+_addresses[i]);
                try
                {
                    Util.sendRequest(_addresses[i],"getstatus".getBytes(),_socket,27960);
                }
                catch (IOException ex)
                {
                    CAT.error(ex);
                }
            }
            Object sleepObj=new Object();
            synchronized (sleepObj)
            {
                try
                {
                    sleepObj.wait(_sleepTime);
                }
                catch (InterruptedException ex) {}
            }
        }
    }

    public void startSingleQuery()
    {
        _socket=de.hendriklipka.util.UDPSocketPool.getSocket();
        try
        {
            _socket.setSoTimeout(1000);
        }
        catch (SocketException e)
        {
            CAT.error(e);
        }
        BroadcastListener bcl=new BroadcastListener(true);
        bcl.start();
        for (int i=0;i<_addresses.length;i++)
        {
            CAT.debug("sending broadcast packet to "+_addresses[i]);
            try
            {
                Util.sendRequest(_addresses[i],"getstatus".getBytes(),_socket,27960);
            }
            catch (IOException ex)
            {
                CAT.error(ex);
            }
        }
        Object sleepObj=new Object();
        synchronized (sleepObj)
        {
            try
            {
                sleepObj.wait(10*1000);
            }
            catch (InterruptedException ex) {}
        }
        bcl.fini=true;
    }

    protected class BroadcastListener extends Thread
    {
        boolean _reportOnly=false;
        public boolean fini=false;

        public BroadcastListener(boolean reportOnly)
        {
            _reportOnly=reportOnly;
        }

        public void run()
        {
            while (!fini)
            {
                try
                {
                    byte[] buffer=new byte[65507]; // maximum packet size
                    DatagramPacket pr=new DatagramPacket(buffer,buffer.length);
                    _socket.receive(pr);
                    InetAddress from=pr.getAddress();
                    int fromPort=pr.getPort();
                    LOG_CAT.info("received broadcast answer from "+from+":"+fromPort);
                    ServerData sd=new ServerData();
                    sd._serverAddress=from;
                    sd._serverPort=fromPort;
                    sd._lastHeartBeat=System.currentTimeMillis();
                    if (_reportOnly)
                    {
                        System.out.println("server found at "+from+":"+fromPort);
                    }
                    else
                    {
                        _handler.addServerByBroadcast(sd);
                    }
                }
                catch (InterruptedIOException ex)
                {
                }
                catch (Exception ex)
                {
                    CAT.error(ex);
                }
            }
        }
    }
}